package com.bowlerhatgames.gridshock
{
   import com.bowlerhatgames.effects.ColorFlash;
   import com.bowlerhatgames.gridshock.assets.soundfx.ActionSound;
   import com.bowlerhatgames.gridshock.assets.soundfx.FinalRowWarningSound;
   import com.bowlerhatgames.gridshock.assets.soundfx.MatchSound;
   import com.bowlerhatgames.gridshock.assets.soundfx.NewLevelSound;
   import com.bowlerhatgames.gridshock.assets.soundfx.NewRowSound;
   import com.gskinner.motion.GTween;
   import com.yahoo.astra.utils.StringUtil;
   import flash.display.MovieClip;
   import flash.display.Sprite;
   import flash.errors.IllegalOperationError;
   import flash.events.Event;
   import flash.events.KeyboardEvent;
   import flash.events.MouseEvent;
   import flash.events.TimerEvent;
   import flash.net.URLRequest;
   import flash.net.navigateToURL;
   import flash.text.TextField;
   import flash.ui.Keyboard;
   import flash.utils.Timer;
   import flash.utils.getTimer;
   import mochi.as3.MochiAd;
   import mochi.as3.MochiScores;
   
   [Embed(source="/_assets/assets.swf", symbol="symbol76")]
   public class GameBoard extends Sprite
   {
      private static const LIGHTS_PER_ROW:int = 12;
      
      private static const SINGLE_MOVE_SCORE:uint = 100;
      
      private static const THREE_ITEM_MATCH_SCORE:uint = 1000;
      
      private static const ADDITIONAL_ITEM_MATCH_SCORE:uint = 500;
      
      private static const ROW_TIMER_DELAY:int = 15000;
      
      private static const PROGRESS_UPDATE_TIMER_DELAY:int = 20;
      
      private static const OBJECTIVES:Array = [10000,25000,50000,100000,150000,200000,300000,400000,550000,700000,850000,1000000];
      
      private var _objectives:Array;
      
      private var _initialized:Boolean = false;
      
      private var _matchTimer:Timer;
      
      private var _newRowTimerStartTime:int;
      
      public var buttonBarLeft:Sprite;
      
      private var _lights:Array;
      
      public var grid:Sprite;
      
      private var _progressBarFullEffect:GTween;
      
      private var _currentColor:uint = 0;
      
      private var _comboPoints:uint;
      
      private var _paused:Boolean = true;
      
      private var _showHideNotification:GTween;
      
      public var actionIndicator:Sprite;
      
      private var _ignoreStateChanges:Boolean = false;
      
      private var _level:int = 1;
      
      public var scoreField:TextField;
      
      private var _score:uint = 0;
      
      private var _progressBarTimer:Timer;
      
      private var _lastMatchSound:int;
      
      public var progressBar:ProgressLight;
      
      private var _nextColor:uint = 0;
      
      private var _activeButton:LightBase = null;
      
      private var _leftButtons:Array;
      
      public var moreGamesButton:Sprite;
      
      public var nextColorLight:LightBase;
      
      private var _newRowTimerSavedTime:int;
      
      private var _progressBarDelay:int;
      
      public var levelField:TextField;
      
      private var _matchCount:int;
      
      public var notification:Notification;
      
      private var _mochiContainer:MovieClip;
      
      public function GameBoard()
      {
         _lights = [];
         _leftButtons = [];
         super();
         this.buttonBarLeft.mouseEnabled = this.buttonBarLeft.mouseChildren = false;
         this.initialize();
      }
      
      private function stageKeyDownHandler(param1:KeyboardEvent) : void
      {
         var _loc2_:int = 0;
         if(!this._initialized || this.paused)
         {
            return;
         }
         switch(param1.keyCode)
         {
            case Keyboard.UP:
               if(!this.activeButton)
               {
                  this.activeButton = LightBase(this._leftButtons[this._leftButtons.length - 1]);
                  break;
               }
               _loc2_ = int(this._leftButtons.indexOf(this._activeButton));
               _loc2_--;
               _loc2_ = Math.max(_loc2_,0);
               this.activeButton = LightBase(this._leftButtons[_loc2_]);
               break;
            case Keyboard.DOWN:
               if(!this.activeButton)
               {
                  this.activeButton = LightBase(this._leftButtons[0]);
                  break;
               }
               _loc2_ = int(this._leftButtons.indexOf(this._activeButton));
               _loc2_++;
               _loc2_ = Math.min(_loc2_,this._leftButtons.length - 1);
               this.activeButton = LightBase(this._leftButtons[_loc2_]);
               break;
            case Keyboard.SPACE:
               if(!this.activeButton)
               {
                  break;
               }
               this.action(this.activeButton);
               break;
         }
      }
      
      public function set level(param1:int) : void
      {
         this._level = param1;
         this.levelField.text = "LEVEL " + this._level.toString();
      }
      
      private function lightButtonRollOverHandler(param1:MouseEvent) : void
      {
         var _loc2_:LightBase = LightBase(param1.currentTarget);
         this.activeButton = _loc2_;
      }
      
      public function set score(param1:uint) : void
      {
         this._score = param1;
         this.scoreField.text = StringUtil.padFront(this._score.toString(),9,"0");
      }
      
      public function get paused() : Boolean
      {
         return this._paused;
      }
      
      public function get nextColor() : uint
      {
         return this._nextColor;
      }
      
      private function matchTimerCompleteHandler(param1:TimerEvent) : void
      {
         this._matchTimer.removeEventListener(TimerEvent.TIMER_COMPLETE,matchTimerCompleteHandler);
         this._matchTimer = null;
         this.hideNotification();
      }
      
      private function action(param1:LightBase) : void
      {
         var _loc7_:int = 0;
         var _loc8_:LightBase = null;
         var _loc9_:int = 0;
         var _loc10_:LightBase = null;
         var _loc2_:int = int(this._leftButtons.indexOf(param1));
         var _loc3_:LightBase = this._lights[LIGHTS_PER_ROW * _loc2_];
         if(_loc3_.currentState != LightBase.STATE_OFF && _loc3_.currentState != LightBase.STATE_FLASH)
         {
            new FinalRowWarningSound().play();
            return;
         }
         var _loc4_:int = this._lights.length / LIGHTS_PER_ROW;
         var _loc5_:int = this._lights.length / _loc4_;
         var _loc6_:int = 0;
         while(_loc6_ < _loc5_)
         {
            _loc7_ = LIGHTS_PER_ROW * _loc2_ + _loc6_;
            _loc8_ = LightBase(this._lights[_loc7_]);
            _loc8_.color = this.currentColor;
            _loc9_ = LIGHTS_PER_ROW * _loc2_ + _loc6_ + 1;
            _loc10_ = this._lights[_loc9_] as LightBase;
            if(_loc6_ == _loc5_ - 1 || !_loc10_ || _loc10_.currentState != LightBase.STATE_OFF && _loc10_.currentState != LightBase.STATE_FLASH)
            {
               _loc8_.currentState = LightBase.STATE_ACTIVATING;
               new ActionSound().play();
               this.score += SINGLE_MOVE_SCORE;
               break;
            }
            _loc8_.currentState = LightBase.STATE_FLASH;
            _loc6_++;
         }
         _loc3_ = this._lights[LIGHTS_PER_ROW * _loc2_];
         if(_loc3_.currentState != LightBase.STATE_OFF && _loc3_.currentState != LightBase.STATE_FLASH)
         {
            new FinalRowWarningSound().play();
         }
         this.currentColor = this.nextColor;
         this.nextColor = this.randomColor();
         param1.color = this.currentColor;
      }
      
      private function lightStateChangeHandler(param1:Event) : void
      {
         if(this._ignoreStateChanges || !this._initialized)
         {
            return;
         }
         var _loc2_:LightBase = LightBase(param1.currentTarget);
         var _loc3_:int = int(this._lights.indexOf(_loc2_));
         if(_loc2_.currentState == LightBase.STATE_ON)
         {
            this.handleMatchesAtIndex(_loc3_);
         }
         this.gravity();
      }
      
      private function progressBarTimerUpdateHandler(param1:TimerEvent) : void
      {
         var _loc2_:Number = (this._newRowTimerSavedTime + (getTimer() - this._newRowTimerStartTime)) / this._progressBarDelay;
         this.progressBar.progress = Math.min(1,_loc2_);
         if(_loc2_ >= 1)
         {
            this._progressBarTimer.stop();
            new NewRowSound().play();
            this._progressBarFullEffect = new ColorFlash(this.progressBar,16777215,1,2);
            this._progressBarFullEffect.addEventListener(Event.COMPLETE,progressBarFullEffectCompleteHandler);
         }
      }
      
      private function handleMatchesAtIndices(param1:Array) : void
      {
         var _loc5_:int = 0;
         var _loc6_:LightBase = null;
         var _loc7_:uint = 0;
         var _loc8_:uint = 0;
         var _loc9_:int = 0;
         var _loc2_:Array = this.getMatchesForIndices(param1);
         var _loc3_:int = int(_loc2_.length);
         var _loc4_:int = 0;
         while(_loc4_ < _loc3_)
         {
            _loc5_ = _loc2_[_loc4_] as int;
            _loc6_ = LightBase(this._lights[_loc5_]);
            _loc6_.currentState = LightBase.STATE_WARNING;
            _loc4_++;
         }
         if(_loc3_ > 0)
         {
            _loc7_ = THREE_ITEM_MATCH_SCORE + (_loc3_ - 3) * ADDITIONAL_ITEM_MATCH_SCORE;
            if(this._matchTimer)
            {
               ++this._matchCount;
               this._matchTimer.reset();
               this._matchTimer.start();
               this.notification.primaryField.text = this._matchCount + "x combo!";
               this.showNotification();
            }
            else
            {
               this._matchCount = 1;
               this._comboPoints = 0;
               this._matchTimer = new Timer(450,1);
               this._matchTimer.addEventListener(TimerEvent.TIMER_COMPLETE,matchTimerCompleteHandler);
               this._matchTimer.start();
            }
            _loc8_ = this._matchCount * _loc7_;
            this._comboPoints += _loc8_;
            this.score += _loc8_;
            this.notification.secondaryField.text = this._comboPoints + " points";
            _loc9_ = getTimer();
            if(_loc9_ - this._lastMatchSound > 250)
            {
               new MatchSound().play();
               this._lastMatchSound = _loc9_;
            }
            this.checkForNextObjective();
         }
      }
      
      private function gravity() : void
      {
         var _loc5_:int = 0;
         var _loc6_:int = 0;
         var _loc7_:LightBase = null;
         var _loc8_:int = 0;
         var _loc9_:int = 0;
         var _loc10_:LightBase = null;
         var _loc1_:Array = [];
         this._ignoreStateChanges = true;
         var _loc2_:int = this._lights.length / LIGHTS_PER_ROW;
         var _loc3_:int = this._lights.length / _loc2_;
         var _loc4_:int = _loc3_ - 1;
         while(_loc4_ >= 0)
         {
            _loc5_ = 0;
            while(_loc5_ < _loc2_)
            {
               _loc6_ = _loc5_ * _loc3_ + _loc4_;
               _loc7_ = LightBase(this._lights[_loc6_]);
               if(_loc7_.currentState == LightBase.STATE_OFF || _loc7_.currentState == LightBase.STATE_FLASH)
               {
                  _loc8_ = _loc4_;
                  while(_loc8_ > 0)
                  {
                     _loc8_--;
                     _loc9_ = _loc5_ * _loc3_ + _loc8_;
                     _loc10_ = LightBase(this._lights[_loc9_]);
                     if(_loc10_.currentState == LightBase.STATE_ON)
                     {
                        _loc7_.copyState(_loc10_);
                        _loc10_.currentState = LightBase.STATE_OFF;
                        _loc1_.push(_loc6_);
                        break;
                     }
                     if(_loc10_.currentState != LightBase.STATE_OFF && _loc10_.currentState != LightBase.STATE_FLASH)
                     {
                        break;
                     }
                  }
               }
               _loc5_++;
            }
            _loc4_--;
         }
         this._ignoreStateChanges = false;
         this.handleMatchesAtIndices(_loc1_);
      }
      
      private function reset() : void
      {
         var _loc1_:LightBase = null;
         for each(_loc1_ in this._lights)
         {
            _loc1_.currentState = LightBase.STATE_OFF;
         }
         if(this._progressBarFullEffect)
         {
            this._progressBarFullEffect.pause();
            this._progressBarFullEffect.removeEventListener(Event.COMPLETE,progressBarFullEffectCompleteHandler);
            this._progressBarFullEffect = null;
         }
         this.score = 0;
         this.level = 1;
         this._objectives = OBJECTIVES.concat();
         this.addRandomRow();
         this.addRandomRow();
         this.currentColor = this.randomColor();
         this.nextColor = this.randomColor();
         this.progressBar.progress = 0;
         this._newRowTimerSavedTime = 0;
         this._progressBarDelay = ROW_TIMER_DELAY;
         this.buttonBarLeft.mouseEnabled = this.buttonBarLeft.mouseChildren = false;
         this.notification.visible = this.notification.visible = false;
         this.notification.alpha = this.notification.alpha = 0;
      }
      
      public function set paused(param1:Boolean) : void
      {
         this._paused = param1;
         if(this._paused)
         {
            this._newRowTimerSavedTime += getTimer() - this._newRowTimerStartTime;
            this._progressBarTimer.stop();
            this.showNotification();
         }
         else
         {
            this._newRowTimerStartTime = getTimer();
            this._progressBarTimer.start();
            this._initialized = true;
            this.hideNotification();
         }
         this.buttonBarLeft.mouseEnabled = this.buttonBarLeft.mouseChildren = !this._paused;
         this.notification.primaryField.text = "PAUSED!";
         this.notification.secondaryField.text = "press p to continue";
      }
      
      public function set nextColor(param1:uint) : void
      {
         this._nextColor = param1;
         this.nextColorLight.animateColor(this._nextColor);
      }
      
      private function checkForNextObjective() : void
      {
         if(this._objectives.length == 0)
         {
            return;
         }
         var _loc1_:uint = this._objectives[0] as uint;
         if(this.score >= _loc1_)
         {
            this._objectives.shift();
            ++this.level;
            this._newRowTimerSavedTime = 0;
            this._newRowTimerStartTime = getTimer();
            this._progressBarDelay *= 0.95;
            new ColorFlash(this.levelField,16777215,1,2);
            new NewLevelSound().play();
         }
      }
      
      private function randomColor() : uint
      {
         var _loc1_:Array = [7254784,11755776,28339,10485939,5767347];
         return _loc1_[int(Math.random() * _loc1_.length)];
      }
      
      public function get currentColor() : uint
      {
         return this._currentColor;
      }
      
      private function hideNotification(param1:Function = null) : void
      {
         if(this._showHideNotification)
         {
            this._showHideNotification.removeEventListener(Event.COMPLETE,notificationShowHideCompleteHandler);
            this._showHideNotification.pause();
            this._showHideNotification = null;
         }
         this._showHideNotification = new GTween(this.notification,0.15,{
            "alpha":0,
            "scaleX":0.5,
            "scaleY":0.5
         });
         this._showHideNotification.addEventListener(Event.COMPLETE,notificationShowHideCompleteHandler);
         if(param1 != null)
         {
            this._showHideNotification.addEventListener(Event.COMPLETE,param1,false,0,true);
         }
      }
      
      private function getMatchesForIndices(param1:Array) : Array
      {
         var _loc5_:int = 0;
         var _loc6_:Array = null;
         var _loc7_:int = 0;
         var _loc8_:int = 0;
         var _loc9_:int = 0;
         var _loc2_:Array = [];
         var _loc3_:int = int(param1.length);
         var _loc4_:int = 0;
         while(_loc4_ < _loc3_)
         {
            _loc5_ = param1[_loc4_] as int;
            _loc6_ = this.getMatchesForIndex(_loc5_);
            _loc7_ = int(_loc6_.length);
            _loc8_ = 0;
            while(_loc8_ < _loc7_)
            {
               _loc9_ = _loc6_[_loc8_] as int;
               if(_loc2_.indexOf(_loc9_) < 0)
               {
                  _loc2_.push(_loc9_);
               }
               _loc8_++;
            }
            _loc4_++;
         }
         return _loc2_;
      }
      
      private function addRandomRow() : void
      {
         var _loc5_:int = 0;
         var _loc6_:int = 0;
         var _loc7_:LightBase = null;
         var _loc8_:int = 0;
         var _loc9_:LightBase = null;
         var _loc10_:uint = 0;
         this._ignoreStateChanges = true;
         var _loc1_:int = this._lights.length / LIGHTS_PER_ROW;
         var _loc2_:int = this._lights.length / _loc1_;
         var _loc3_:uint = 0;
         var _loc4_:int = 0;
         while(_loc4_ < _loc2_)
         {
            _loc5_ = 0;
            while(_loc5_ < _loc1_)
            {
               _loc6_ = _loc5_ * _loc2_ + _loc4_;
               _loc7_ = LightBase(this._lights[_loc6_]);
               if(_loc4_ == 0)
               {
                  if(_loc7_.currentState != LightBase.STATE_OFF && _loc7_.currentState != LightBase.STATE_FLASH)
                  {
                     new ColorFlash(this.root,16777215,1,2);
                     this.gameOver();
                     return;
                  }
               }
               if(_loc4_ < _loc2_ - 1)
               {
                  _loc8_ = _loc5_ * _loc2_ + (_loc4_ + 1);
                  _loc9_ = LightBase(this._lights[_loc8_]);
                  _loc7_.copyState(_loc9_);
                  if(_loc4_ == 0 && _loc7_.currentState != LightBase.STATE_OFF && _loc7_.currentState != LightBase.STATE_FLASH)
                  {
                     new FinalRowWarningSound().play();
                  }
               }
               else
               {
                  _loc10_ = _loc3_;
                  while(_loc10_ == _loc3_)
                  {
                     _loc10_ = this.randomColor();
                  }
                  _loc7_.color = _loc10_;
                  _loc3_ = _loc10_;
                  _loc7_.currentState = LightBase.STATE_OFF;
                  _loc7_.currentState = LightBase.STATE_ACTIVATING;
               }
               _loc5_++;
            }
            _loc4_++;
         }
         this._ignoreStateChanges = false;
      }
      
      private function moreGamesButtonClickHandler(param1:MouseEvent) : void
      {
         navigateToURL(new URLRequest(GlobalSettings.MORE_GAMES_URL),"_blank");
      }
      
      private function lightButtonRollOutHandler(param1:MouseEvent) : void
      {
         if(this.activeButton == param1.currentTarget)
         {
            this.activeButton = null;
         }
      }
      
      public function get level() : int
      {
         return this._level;
      }
      
      private function notificationShowHideCompleteHandler(param1:Event) : void
      {
         this._showHideNotification.pause();
         this._showHideNotification.removeEventListener(Event.COMPLETE,notificationShowHideCompleteHandler);
         this._showHideNotification = null;
      }
      
      private function findButtons() : void
      {
         var _loc3_:LightBase = null;
         var _loc1_:int = this.buttonBarLeft.numChildren;
         var _loc2_:int = 0;
         while(_loc2_ < _loc1_)
         {
            _loc3_ = this.buttonBarLeft.getChildAt(_loc2_) as LightBase;
            if(_loc3_)
            {
               _loc3_.currentState = LightBase.STATE_OFF;
               _loc3_.addEventListener(MouseEvent.ROLL_OVER,lightButtonRollOverHandler);
               this._leftButtons.push(_loc3_);
            }
            _loc2_++;
         }
      }
      
      private function initialize() : void
      {
         this.findButtons();
         this.findGridLights();
         this.notification.mouseEnabled = this.notification.mouseChildren = false;
         this.notification.scaleX = this.notification.scaleY = 0.5;
         this.moreGamesButton.buttonMode = this.moreGamesButton.useHandCursor = true;
         this.moreGamesButton.addEventListener(MouseEvent.CLICK,moreGamesButtonClickHandler);
         this._progressBarTimer = new Timer(PROGRESS_UPDATE_TIMER_DELAY);
         this._progressBarTimer.addEventListener(TimerEvent.TIMER,progressBarTimerUpdateHandler);
         this.progressBar.currentState = LightBase.STATE_ON;
         this.nextColorLight.currentState = LightBase.STATE_ON;
         this._mochiContainer = new MovieClip();
         this.addChild(this._mochiContainer);
         this.reset();
         this.addEventListener(Event.ADDED_TO_STAGE,addedToStageHandler);
      }
      
      public function get score() : uint
      {
         return this._score;
      }
      
      private function findGridLights() : void
      {
         var _loc3_:LightBase = null;
         var _loc1_:int = this.grid.numChildren;
         var _loc2_:int = 0;
         while(_loc2_ < _loc1_)
         {
            _loc3_ = this.grid.getChildAt(_loc2_) as LightBase;
            if(_loc3_)
            {
               _loc3_.addEventListener(Event.CHANGE,lightStateChangeHandler);
               this._lights.push(_loc3_);
            }
            _loc2_++;
         }
      }
      
      private function showNotification(param1:Function = null) : void
      {
         if(this._showHideNotification)
         {
            this._showHideNotification.removeEventListener(Event.COMPLETE,notificationShowHideCompleteHandler);
            this._showHideNotification.pause();
            this._showHideNotification = null;
         }
         this._showHideNotification = new GTween(this.notification,0.15,{
            "alpha":1,
            "scaleX":1,
            "scaleY":1
         });
         this._showHideNotification.addEventListener(Event.COMPLETE,notificationShowHideCompleteHandler);
         if(param1 != null)
         {
            this._showHideNotification.addEventListener(Event.COMPLETE,param1,false,0,true);
         }
      }
      
      private function addedToStageHandler(param1:Event) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         this.stage.addEventListener(KeyboardEvent.KEY_UP,stageKeyUpHandler);
         this.stage.addEventListener(KeyboardEvent.KEY_DOWN,stageKeyDownHandler);
         this.paused = false;
      }
      
      private function gameOverClickHandler(param1:MouseEvent) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         var _loc3_:Object = {
            "id":"38ef553de9d1dbe6",
            "clip":this._mochiContainer,
            "res":"640x480",
            "color":8978176,
            "background":0,
            "outline":8960768,
            "ad_finished":interLevelAdComplete
         };
         MochiAd.showInterLevelAd(_loc3_);
         this.hideNotification();
      }
      
      private function progressBarFullEffectCompleteHandler(param1:Event) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         this._progressBarFullEffect = null;
         this.addRandomRow();
         this._newRowTimerSavedTime = 0;
         if(!this._paused)
         {
            this._newRowTimerStartTime = getTimer();
            this._progressBarTimer.start();
         }
      }
      
      private function getMatchesForIndex(param1:int) : Array
      {
         var _loc5_:int = 0;
         var _loc6_:LightBase = null;
         var _loc7_:uint = 0;
         var _loc8_:int = 0;
         var _loc9_:int = 0;
         var _loc10_:LightBase = null;
         var _loc11_:int = 0;
         var _loc12_:LightBase = null;
         var _loc13_:LightBase = null;
         var _loc14_:int = 0;
         var _loc15_:LightBase = null;
         var _loc2_:Array = [];
         var _loc3_:Array = [param1];
         while(_loc3_.length > 0)
         {
            _loc5_ = _loc3_.shift() as int;
            _loc6_ = LightBase(this._lights[_loc5_]);
            if(_loc6_.currentState != LightBase.STATE_ON)
            {
               throw new IllegalOperationError("Cannot find matches for a light that is not on.");
            }
            _loc7_ = _loc6_.color;
            _loc8_ = _loc5_ - LIGHTS_PER_ROW;
            if(_loc8_ >= 0)
            {
               _loc10_ = LightBase(this._lights[_loc8_]);
               if(_loc2_.indexOf(_loc8_) < 0 && _loc3_.indexOf(_loc8_) < 0 && _loc10_.currentState == LightBase.STATE_ON && _loc10_.color == _loc7_)
               {
                  _loc3_.push(_loc8_);
               }
            }
            if(_loc5_ % LIGHTS_PER_ROW < LIGHTS_PER_ROW - 1)
            {
               _loc11_ = _loc5_ + 1;
               _loc12_ = LightBase(this._lights[_loc11_]);
               if(_loc2_.indexOf(_loc11_) < 0 && _loc3_.indexOf(_loc11_) < 0 && _loc12_.currentState == LightBase.STATE_ON && _loc12_.color == _loc7_)
               {
                  _loc3_.push(_loc11_);
               }
            }
            _loc9_ = _loc5_ + LIGHTS_PER_ROW;
            if(_loc9_ < this._lights.length)
            {
               _loc13_ = LightBase(this._lights[_loc9_]);
               if(_loc2_.indexOf(_loc9_) < 0 && _loc3_.indexOf(_loc9_) < 0 && _loc13_.currentState == LightBase.STATE_ON && _loc13_.color == _loc7_)
               {
                  _loc3_.push(_loc9_);
               }
            }
            if(_loc5_ % LIGHTS_PER_ROW > 0)
            {
               _loc14_ = _loc5_ - 1;
               _loc15_ = LightBase(this._lights[_loc14_]);
               if(_loc2_.indexOf(_loc14_) < 0 && _loc3_.indexOf(_loc14_) < 0 && _loc15_.currentState == LightBase.STATE_ON && _loc15_.color == _loc7_)
               {
                  _loc3_.push(_loc14_);
               }
            }
            _loc2_.push(_loc5_);
         }
         var _loc4_:int = int(_loc2_.length);
         if(_loc4_ >= 3)
         {
            return _loc2_;
         }
         return [];
      }
      
      private function stageKeyUpHandler(param1:KeyboardEvent) : void
      {
         if(!this._initialized)
         {
            return;
         }
         var _loc2_:String = String.fromCharCode(param1.charCode).toLowerCase();
         if(_loc2_ == "p")
         {
            this.paused = !this.paused;
         }
         else if(_loc2_ == "s")
         {
            GlobalSettings.defaultSettings.soundsOff = !GlobalSettings.defaultSettings.soundsOff;
         }
         else if(_loc2_ == "r")
         {
            this.reset();
            this.paused = false;
         }
      }
      
      public function set activeButton(param1:LightBase) : void
      {
         if(this._activeButton)
         {
            this.deactivateLightButton(this._activeButton);
            this._activeButton = null;
         }
         this._activeButton = param1;
         if(this._activeButton)
         {
            this.activateLightButton(this._activeButton);
         }
      }
      
      private function interLevelAdComplete() : void
      {
         this.reset();
         this.paused = false;
      }
      
      private function matchAnimationCompleteHandler(param1:Event) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         var _loc3_:GTween = GTween(param1.currentTarget);
         var _loc4_:LightBase = LightBase(_loc3_.target);
         _loc4_.currentState = LightBase.STATE_DEACTIVATING;
      }
      
      private function lightButtonClickHandler(param1:MouseEvent) : void
      {
         var _loc2_:LightBase = LightBase(param1.currentTarget);
         this.action(_loc2_);
      }
      
      public function set currentColor(param1:uint) : void
      {
         this._currentColor = param1;
         this.progressBar.animateColor(this._currentColor);
      }
      
      public function get activeButton() : LightBase
      {
         return this._activeButton;
      }
      
      private function deactivateLightButton(param1:LightBase) : void
      {
         param1.removeEventListener(MouseEvent.ROLL_OUT,lightButtonRollOutHandler);
         param1.removeEventListener(MouseEvent.CLICK,lightButtonClickHandler);
         param1.currentState = LightBase.STATE_DEACTIVATING;
         param1.buttonMode = false;
      }
      
      private function gameOver() : void
      {
         var boardID:String;
         var o:Object;
         this._initialized = false;
         this.paused = true;
         this.notification.primaryField.text = "GAME OVER!";
         this.notification.secondaryField.text = "click to play again";
         this.hideNotification();
         o = {
            "n":[5,4,7,8,12,12,15,6,13,9,7,8,10,8,0,11],
            "f":function(param1:Number, param2:String):String
            {
               if(param2.length == 16)
               {
                  return param2;
               }
               return this.f(param1 + 1,param2 + this.n[param1].toString(16));
            }
         };
         boardID = o.f(0,"");
         MochiScores.showLeaderboard({
            "boardID":boardID,
            "score":this.score,
            "res":"640x480",
            "onClose":function():void
            {
               showNotification();
               addEventListener(MouseEvent.CLICK,gameOverClickHandler);
            }
         });
      }
      
      private function handleMatchesAtIndex(param1:int) : void
      {
         this.handleMatchesAtIndices([param1]);
      }
      
      private function activateLightButton(param1:LightBase) : void
      {
         param1.addEventListener(MouseEvent.CLICK,lightButtonClickHandler);
         param1.addEventListener(MouseEvent.ROLL_OUT,lightButtonRollOutHandler);
         param1.color = this.currentColor;
         param1.currentState = LightBase.STATE_ACTIVATING;
         param1.buttonMode = true;
      }
   }
}

