package com.bowlerhatgames.gridshock
{
   import com.bowlerhatgames.effects.ColorFlash;
   import com.gskinner.motion.GTween;
   import flash.display.Shape;
   import flash.display.Sprite;
   import flash.errors.IllegalOperationError;
   import flash.events.Event;
   import flash.geom.ColorTransform;
   import org.josht.utils.ColorUtil;
   
   public class LightBase extends Sprite
   {
      public static const STATE_ACTIVATING:String = "activating";
      
      public static const STATE_DEACTIVATING:String = "deactivating";
      
      public static const STATE_ON:String = "on";
      
      public static const STATE_OFF:String = "off";
      
      public static const STATE_FLASH:String = "flash";
      
      public static const STATE_WARNING:String = "warning";
      
      private var _color:uint = 0;
      
      private var _colorTween:GTween;
      
      public var base:Sprite;
      
      private var _warningTween:GTween;
      
      private var _currentState:String = "off";
      
      private var _baseInternalAlphaTween:GTween;
      
      protected var baseInternal:Shape;
      
      public function LightBase()
      {
         super();
         this.baseInternal = Shape(this.base.getChildAt(0));
         this.baseInternal.alpha = 0;
         this.baseInternal.visible = false;
      }
      
      private function colorTweenCompleteHandler(param1:Event) : void
      {
         this._colorTween.pause();
         this._colorTween.removeEventListener(Event.COMPLETE,colorTweenCompleteHandler);
         this._colorTween = null;
      }
      
      public function animateColor(param1:uint) : void
      {
         if(this._colorTween)
         {
            this._colorTween.pause();
            this._colorTween.removeEventListener(Event.COMPLETE,colorTweenCompleteHandler);
            this._colorTween = null;
         }
         this._color = param1;
         var _loc2_:uint = ColorUtil.red(this._color);
         var _loc3_:uint = ColorUtil.green(this._color);
         var _loc4_:uint = ColorUtil.blue(this._color);
         this._colorTween = new GTween(this.base,0.25,{
            "redOffset":_loc2_,
            "greenOffset":_loc3_,
            "blueOffset":_loc4_
         },{"completeListener":colorTweenCompleteHandler});
         this._colorTween.setAssignment(this.base.transform,"colorTransform");
      }
      
      public function get color() : uint
      {
         return this._color;
      }
      
      public function copyState(param1:LightBase) : void
      {
         var _loc2_:String = param1.currentState == STATE_FLASH ? STATE_OFF : param1.currentState;
         this.color = param1.color;
         this.baseInternal.visible = param1.baseInternal.visible;
         this.baseInternal.alpha = param1.baseInternal.alpha;
         this.currentState = _loc2_;
      }
      
      public function set color(param1:uint) : void
      {
         if(this._colorTween)
         {
            this._colorTween.pause();
            this._colorTween.removeEventListener(Event.COMPLETE,colorTweenCompleteHandler);
            this._colorTween = null;
         }
         this._color = param1;
         var _loc2_:uint = ColorUtil.red(this._color);
         var _loc3_:uint = ColorUtil.green(this._color);
         var _loc4_:uint = ColorUtil.blue(this._color);
         this.base.transform.colorTransform = new ColorTransform(1,1,1,1,_loc2_,_loc3_,_loc4_,0);
      }
      
      private function flashTweenCompleteListener(param1:Event) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         this._baseInternalAlphaTween = this._baseInternalAlphaTween.nextTween;
         if(this._baseInternalAlphaTween)
         {
            this._baseInternalAlphaTween.addEventListener(Event.COMPLETE,flashTweenCompleteListener);
         }
         else
         {
            this.currentState = STATE_OFF;
         }
      }
      
      public function set currentState(param1:String) : void
      {
         var _loc3_:Number = NaN;
         var _loc4_:Number = NaN;
         var _loc2_:String = this._currentState;
         this._currentState = param1;
         if(this._baseInternalAlphaTween)
         {
            this._baseInternalAlphaTween.pause();
            this._baseInternalAlphaTween.nextTween = null;
            this._baseInternalAlphaTween.removeEventListener(Event.COMPLETE,flashTweenCompleteListener);
            this._baseInternalAlphaTween.removeEventListener(Event.COMPLETE,activationTweenCompleteListener);
            this._baseInternalAlphaTween = null;
         }
         if(this._warningTween)
         {
            this._warningTween.pause();
            this._warningTween.removeEventListener(Event.COMPLETE,warningTweenCompleteHandler);
            this._warningTween = null;
         }
         this.transform.colorTransform = new ColorTransform();
         switch(this._currentState)
         {
            case STATE_ON:
               this.baseInternal.alpha = 1;
               this.baseInternal.visible = true;
               break;
            case STATE_OFF:
               this.baseInternal.alpha = 0;
               this.baseInternal.visible = false;
               break;
            case STATE_WARNING:
               this.baseInternal.alpha = 1;
               this.baseInternal.visible = true;
               this._warningTween = new ColorFlash(this,16750848,1,3);
               this._warningTween.addEventListener(Event.COMPLETE,warningTweenCompleteHandler);
               break;
            case STATE_FLASH:
               if(_loc2_ != STATE_OFF && _loc2_ != STATE_FLASH)
               {
                  throw new IllegalOperationError("Can\'t flash a light that\'s on!");
               }
               this.baseInternal.alpha = 0;
               this.baseInternal.visible = false;
               this._baseInternalAlphaTween = new GTween(this.baseInternal,0.1,{"alpha":1});
               this._baseInternalAlphaTween.addEventListener(Event.COMPLETE,flashTweenCompleteListener);
               this._baseInternalAlphaTween.nextTween = new GTween(this.baseInternal,0.25,{"alpha":0},{"paused":true});
               break;
            default:
               _loc3_ = this._currentState == STATE_ACTIVATING ? 1 : 0;
               _loc4_ = 0.2;
               this._baseInternalAlphaTween = new GTween(this.baseInternal,_loc4_,{"alpha":_loc3_});
               this._baseInternalAlphaTween.addEventListener(Event.COMPLETE,activationTweenCompleteListener);
         }
         this.dispatchEvent(new Event(Event.CHANGE));
      }
      
      private function warningTweenCompleteHandler(param1:Event) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         this._warningTween = null;
         this.currentState = LightBase.STATE_OFF;
      }
      
      public function get currentState() : String
      {
         return this._currentState;
      }
      
      private function activationTweenCompleteListener(param1:Event) : void
      {
         param1.currentTarget.removeEventListener(param1.type,arguments.callee);
         this._baseInternalAlphaTween = null;
         if(this._currentState == LightBase.STATE_ACTIVATING)
         {
            this.currentState = LightBase.STATE_ON;
         }
         else
         {
            if(this._currentState != LightBase.STATE_DEACTIVATING)
            {
               throw new IllegalOperationError("Invalid light state. Animation was not properly removed.");
            }
            this.currentState = LightBase.STATE_OFF;
         }
      }
   }
}

