package com.gskinner.motion
{
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import flash.events.IEventDispatcher;
   import flash.utils.Dictionary;
   
   public class GTween extends EventDispatcher
   {
      protected static var _activeTicker:ITicker;
      
      public static var defaultEase:Function;
      
      protected static var _timingMode:String;
      
      protected static var _timeInterval:uint = 40;
      
      protected static var activeTweens:Dictionary = new Dictionary();
      
      public static const TIME:String = "time";
      
      public static const FRAME:String = "frame";
      
      public static const HYBRID:String = "hybrid";
      
      public static const START:String = "start";
      
      public static const DELAY:String = "delay";
      
      public static const TWEEN:String = "tween";
      
      public static const END:String = "end";
      
      public static var pauseAll:Boolean = false;
      
      public static var rotationProperties:Object = {
         "rotation":true,
         "rotationX":true,
         "rotationY":true,
         "rotationZ":true
      };
      
      public static var snappingProperties:Object = {
         "x":true,
         "y":true
      };
      
      public var autoPlay:Boolean = true;
      
      protected var _position:Number = 0;
      
      protected var ticker:ITicker;
      
      protected var _delay:Number = 0;
      
      public var duration:Number = 1;
      
      public var repeat:int = 0;
      
      protected var positionOffset:Number;
      
      protected var _previousPosition:Number;
      
      protected var startValues:Object;
      
      public var data:*;
      
      protected var inited:Boolean;
      
      protected var _target:Object;
      
      protected var _tweenPosition:Number = 0;
      
      protected var _reversed:Boolean;
      
      public var snapping:Boolean = false;
      
      public var reflect:Boolean = false;
      
      public var nextTween:GTween;
      
      protected var assignmentProperty:String;
      
      public var autoVisible:Boolean = true;
      
      protected var assignmentTarget:Object;
      
      protected var _lockStartProperties:Boolean;
      
      public var autoRotation:Boolean = false;
      
      protected var endValues:Object;
      
      protected var inTick:Boolean;
      
      protected var _paused:Boolean = true;
      
      protected var _propertyTarget:Object;
      
      public var ease:Function;
      
      protected var _proxy:TargetProxy;
      
      protected var _previousTweenPosition:Number;
      
      public function GTween(param1:Object = null, param2:Number = 10, param3:Object = null, param4:Object = null)
      {
         ease = linearEase;
         super();
         ticker = activeTicker;
         this.target = param1;
         this.duration = param2;
         ease = defaultEase || linearEase;
         setProperties(param3);
         setTweenProperties(param4);
      }
      
      public static function set timeInterval(param1:uint) : void
      {
         _timeInterval = param1;
         if(_activeTicker is TimeTicker)
         {
            (_activeTicker as TimeTicker).interval = _timeInterval / 1000;
         }
      }
      
      public static function get timingMode() : String
      {
         return _timingMode;
      }
      
      public static function get activeTicker() : ITicker
      {
         if(_timingMode == null)
         {
            timingMode = HYBRID;
         }
         return _activeTicker;
      }
      
      public static function set timingMode(param1:String) : void
      {
         param1 = param1 == FRAME || param1 == TIME ? param1 : HYBRID;
         if(param1 == _timingMode)
         {
            return;
         }
         _timingMode = param1;
         if(_timingMode == TIME)
         {
            _activeTicker = new TimeTicker();
            (_activeTicker as TimeTicker).interval = _timeInterval / 1000;
         }
         else if(_timingMode == FRAME)
         {
            _activeTicker = new FrameTicker();
         }
         else
         {
            _activeTicker = new HybridTicker();
         }
      }
      
      public static function linearEase(param1:Number, param2:Number, param3:Number, param4:Number) : Number
      {
         return param1;
      }
      
      public static function get timeInterval() : uint
      {
         return _timeInterval;
      }
      
      protected function setGCLock(param1:Boolean) : void
      {
         if(param1)
         {
            if(_target is IEventDispatcher)
            {
               _target.addEventListener("GDS__NONEXISTENT_EVENT",nullListener,false,0,false);
            }
            else
            {
               activeTweens[this] = true;
            }
         }
         else
         {
            if(_target is IEventDispatcher)
            {
               _target.removeEventListener("GDS__NONEXISTENT_EVENT",nullListener);
            }
            delete activeTweens[this];
         }
      }
      
      public function get target() : Object
      {
         return _target;
      }
      
      public function get delay() : Number
      {
         return _delay;
      }
      
      public function set delay(param1:Number) : void
      {
         if(_position == -_delay)
         {
            setPosition(-param1);
         }
         _delay = param1;
      }
      
      protected function init() : void
      {
         var _loc1_:String = null;
         var _loc2_:Number = NaN;
         var _loc3_:Number = NaN;
         inited = true;
         if(_lockStartProperties)
         {
            return;
         }
         startValues = {};
         if(Boolean(assignmentTarget) && Boolean(assignmentProperty))
         {
            _propertyTarget = assignmentTarget[assignmentProperty];
         }
         for(_loc1_ in endValues)
         {
            if(autoRotation && Boolean(rotationProperties[_loc1_]))
            {
               _loc2_ = endValues[_loc1_] = endValues[_loc1_] % 360;
               _loc3_ = _propertyTarget[_loc1_] % 360;
               startValues[_loc1_] = _loc3_ + (Math.abs(_loc3_ - _loc2_) < 180 ? 0 : (_loc3_ > _loc2_ ? -360 : 360));
            }
            else
            {
               startValues[_loc1_] = _propertyTarget[_loc1_];
            }
         }
      }
      
      public function get state() : String
      {
         return _position == -_delay && _paused ? START : (_position < 0 ? DELAY : (repeat != -1 && _position >= (repeat + 1) * duration ? END : TWEEN));
      }
      
      public function get lockStartProperties() : Boolean
      {
         return _lockStartProperties;
      }
      
      public function deleteProperty(param1:String) : Boolean
      {
         return delete endValues[param1];
      }
      
      public function set target(param1:Object) : void
      {
         _propertyTarget = _target = param1 === null ? {} : param1;
         inited = false;
      }
      
      public function set reversed(param1:Boolean) : void
      {
         if(param1 == _reversed)
         {
            return;
         }
         _reversed = param1;
         if(!inited)
         {
            init();
         }
         setPosition(_position,true);
      }
      
      public function get position() : Number
      {
         return _position;
      }
      
      public function setPosition(param1:Number, param2:Boolean = true) : void
      {
         var _loc4_:Number = NaN;
         _previousPosition = _position;
         _position = param1;
         if(!inTick && !paused)
         {
            updatePositionOffset();
         }
         var _loc3_:Number = (repeat + 1) * duration;
         if(param1 < 0)
         {
            _loc4_ = _reversed ? duration : 0;
         }
         else if(repeat == -1 || param1 < _loc3_)
         {
            _loc4_ = param1 % duration;
            if((reflect && param1 / duration % 2 >= 1) != _reversed)
            {
               _loc4_ = duration - _loc4_;
            }
         }
         else
         {
            _loc4_ = (reflect && repeat % 2 >= 1) != _reversed ? 0 : duration;
         }
         if(_loc4_ == _tweenPosition)
         {
            return;
         }
         _previousTweenPosition = _tweenPosition;
         _tweenPosition = _loc4_;
         if(!param2 && hasEventListener(Event.CHANGE))
         {
            dispatchEvent(new Event(Event.CHANGE));
         }
         if(!inited && _previousPosition <= 0 && _position >= 0)
         {
            init();
            if(!param2 && hasEventListener(Event.INIT))
            {
               dispatchEvent(new Event(Event.INIT));
            }
         }
         updateProperties();
         if(repeat != -1 && _previousPosition < _loc3_ && param1 >= _loc3_)
         {
            if(!param2 && hasEventListener(Event.COMPLETE))
            {
               dispatchEvent(new Event(Event.COMPLETE));
            }
            paused = true;
            if(nextTween)
            {
               nextTween.paused = false;
            }
         }
      }
      
      public function set lockStartProperties(param1:Boolean) : void
      {
         if(param1 && !inited)
         {
            init();
         }
         _lockStartProperties = param1;
      }
      
      public function get paused() : Boolean
      {
         return _paused;
      }
      
      public function get tweenPosition() : Number
      {
         return _tweenPosition;
      }
      
      public function invalidate() : void
      {
         inited = false;
         if(_position > 0)
         {
            _position = 0;
            updatePositionOffset();
         }
         if(autoPlay)
         {
            paused = false;
         }
      }
      
      public function getProperties() : Object
      {
         return copyObject(endValues);
      }
      
      public function get propertyTarget() : Object
      {
         return _propertyTarget;
      }
      
      public function play() : void
      {
         paused = false;
      }
      
      public function setAssignment(param1:Object = null, param2:String = null) : void
      {
         this.assignmentTarget = param1;
         this.assignmentProperty = param2;
         inited = false;
      }
      
      public function get reversed() : Boolean
      {
         return _reversed;
      }
      
      public function set position(param1:Number) : void
      {
         setPosition(param1,true);
      }
      
      protected function updateProperty(param1:String, param2:Number, param3:Number, param4:Number) : void
      {
         var _loc5_:Number = param2 + (param3 - param2) * param4;
         if(snapping && Boolean(snappingProperties[param1]))
         {
            _loc5_ = Math.round(_loc5_);
         }
         if(param1 == "currentFrame")
         {
            _propertyTarget.gotoAndStop(_loc5_ << 0);
         }
         else
         {
            _propertyTarget[param1] = _loc5_;
         }
      }
      
      public function setProperties(param1:Object) : void
      {
         var _loc2_:String = null;
         endValues = {};
         for(_loc2_ in param1)
         {
            setProperty(_loc2_,param1[_loc2_]);
         }
      }
      
      public function getProperty(param1:String) : Number
      {
         return endValues[param1];
      }
      
      public function end() : void
      {
         setPosition(repeat == -1 ? duration : (repeat + 1) * duration);
      }
      
      public function set paused(param1:Boolean) : void
      {
         if(param1 == _paused)
         {
            return;
         }
         _paused = param1;
         if(param1)
         {
            ticker.removeEventListener("tick",handleTick);
         }
         else
         {
            ticker.addEventListener("tick",handleTick,false,0,true);
            if(repeat != -1 && _position >= duration * (repeat + 1))
            {
               position = 0;
            }
            else
            {
               updatePositionOffset();
            }
         }
         setGCLock(!param1);
      }
      
      public function beginning() : void
      {
         setPosition(-_delay);
      }
      
      public function setProperty(param1:String, param2:Number) : void
      {
         if(isNaN(param2))
         {
            return;
         }
         endValues[param1] = param2;
         if(_lockStartProperties && startValues[param1] == null)
         {
            startValues[param1] = _propertyTarget[param1];
         }
         invalidate();
      }
      
      protected function updatePositionOffset() : void
      {
         positionOffset = ticker.position - _position;
      }
      
      protected function copyObject(param1:Object) : Object
      {
         var _loc3_:String = null;
         var _loc2_:Object = {};
         for(_loc3_ in param1)
         {
            _loc2_[param1] = param1[_loc3_];
         }
         return _loc2_;
      }
      
      public function setTweenProperties(param1:Object) : void
      {
         var _loc2_:Number = NaN;
         var _loc3_:String = null;
         if(!param1)
         {
            return;
         }
         if("position" in param1)
         {
            _loc2_ = Number(param1.position);
            delete param1.position;
         }
         if("initListener" in param1)
         {
            addEventListener(Event.INIT,param1.initListener,false,0,true);
            delete param1.initListener;
         }
         if("completeListener" in param1)
         {
            addEventListener(Event.COMPLETE,param1.completeListener,false,0,true);
            delete param1.completeListener;
         }
         if("changeListener" in param1)
         {
            addEventListener(Event.CHANGE,param1.changeListener,false,0,true);
            delete param1.changeListener;
         }
         for(_loc3_ in param1)
         {
            this[_loc3_] = param1[_loc3_];
         }
         if(!isNaN(_loc2_))
         {
            position = _loc2_;
         }
      }
      
      protected function updateProperties() : void
      {
         var _loc2_:String = null;
         var _loc1_:Number = ease(_tweenPosition / duration,0,1,1);
         for(_loc2_ in endValues)
         {
            updateProperty(_loc2_,startValues[_loc2_],endValues[_loc2_],_loc1_);
         }
         if(autoVisible && "alpha" in endValues && "alpha" in _propertyTarget && "visible" in _propertyTarget)
         {
            _propertyTarget.visible = _propertyTarget.alpha > 0;
         }
         if(Boolean(assignmentTarget) && Boolean(assignmentProperty))
         {
            assignmentTarget[assignmentProperty] = _propertyTarget;
         }
      }
      
      public function reverse(param1:Boolean = true) : void
      {
         var _loc2_:Number = repeat == -1 ? duration - _position % duration : (repeat + 1) * duration - _position;
         if(reflect)
         {
            _reversed = position / duration % 2 >= 1 == _loc2_ / duration % 2 >= 1 != _reversed;
         }
         else
         {
            _reversed = !_reversed;
         }
         setPosition(_loc2_,param1);
      }
      
      public function setStartProperties(param1:Object) : void
      {
         startValues = copyObject(param1);
         inited = true;
      }
      
      protected function handleTick(param1:Event) : void
      {
         if(_paused)
         {
            return;
         }
         inTick = true;
         if(pauseAll)
         {
            updatePositionOffset();
         }
         else
         {
            setPosition(ticker.position - positionOffset,false);
         }
         inTick = false;
      }
      
      public function get proxy() : Object
      {
         if(_proxy == null)
         {
            _proxy = new TargetProxy(this);
         }
         return _proxy;
      }
      
      public function pause() : void
      {
         paused = true;
      }
      
      protected function nullListener(param1:Event) : void
      {
      }
      
      public function getStartProperties() : Object
      {
         return copyObject(startValues);
      }
   }
}

import flash.display.Shape;
import flash.events.Event;
import flash.events.EventDispatcher;
import flash.events.IEventDispatcher;
import flash.events.TimerEvent;
import flash.utils.Proxy;
import flash.utils.Timer;
import flash.utils.flash_proxy;
import flash.utils.getTimer;

use namespace flash_proxy;

class HybridTicker extends EventDispatcher implements ITicker
{
   protected var shape:Shape;
   
   public function HybridTicker()
   {
      super();
      shape = new Shape();
      shape.addEventListener(Event.ENTER_FRAME,tick);
   }
   
   public function get position() : Number
   {
      return getTimer() / 1000;
   }
   
   protected function tick(param1:Event) : void
   {
      dispatchEvent(new Event("tick"));
   }
}

class TimeTicker extends EventDispatcher implements ITicker
{
   protected var timer:Timer;
   
   public function TimeTicker()
   {
      super();
      timer = new Timer(20);
      timer.addEventListener(TimerEvent.TIMER,tick);
      timer.start();
   }
   
   public function get position() : Number
   {
      return getTimer() / 1000;
   }
   
   public function set interval(param1:Number) : void
   {
      timer.delay = param1 * 1000;
   }
   
   protected function tick(param1:TimerEvent) : void
   {
      dispatchEvent(new Event("tick"));
      param1.updateAfterEvent();
   }
}

interface ITicker extends IEventDispatcher
{
   function get position() : Number;
}

class FrameTicker extends EventDispatcher implements ITicker
{
   protected var shape:Shape;
   
   protected var _position:Number = 0;
   
   public function FrameTicker()
   {
      super();
      shape = new Shape();
      shape.addEventListener(Event.ENTER_FRAME,tick);
   }
   
   public function get position() : Number
   {
      return _position;
   }
   
   protected function tick(param1:Event) : void
   {
      ++_position;
      dispatchEvent(new Event("tick"));
   }
}

dynamic class TargetProxy extends Proxy
{
   private var gTween:GTween;
   
   public function TargetProxy(param1:GTween)
   {
      super();
      this.gTween = param1;
   }
   
   override flash_proxy function deleteProperty(param1:*) : Boolean
   {
      return gTween.deleteProperty(param1);
   }
   
   override flash_proxy function callProperty(param1:*, ... rest) : *
   {
      return gTween.propertyTarget[param1].apply(null,rest);
   }
   
   override flash_proxy function setProperty(param1:*, param2:*) : void
   {
      if(isNaN(param2))
      {
         gTween.propertyTarget[param1] = param2;
      }
      else
      {
         gTween.setProperty(String(param1),Number(param2));
      }
   }
   
   override flash_proxy function getProperty(param1:*) : *
   {
      var _loc2_:Number = Number(gTween.getProperty(param1));
      return isNaN(_loc2_) ? gTween.propertyTarget[param1] : _loc2_;
   }
}
